from django.core.management.base import BaseCommand
from django.conf import settings
from django.contrib.sites.models import Site

from django_url_shortener.piwik import parse_cookie
from django_url_shortener.models import Visit
import requests
import json
import random
try:
    from urllib import urlencode
except ImportError:
    from urllib.parse import urlencode

CURRENT_DOMAIN = Site.objects.get_current().domain


class Command(BaseCommand):
    def handle(self, *args, **options):
        visits = []
        num_visits = Visit.objects.filter(tracked=False).count()
        print('Untracked views: %d' % num_visits)
        if num_visits == 0:
            return

        for visit in Visit.objects.filter(tracked=False).prefetch_related('url'):
            url = visit.url
            params = parse_cookie(visit.cookie)
            if url.campaign_name:
                visited = urlencode({'pk_campaign': url.campaign_name, 'pk_kwd': url.campaign_keyword})
                address = 'http://%s/%s?%s' % (CURRENT_DOMAIN, url.slug, visited)
            else:
                address = 'http://%s/%s' % (CURRENT_DOMAIN, url.slug)
            params.update({'rand': random.randint(0, 1000000), 'url': address, 'urlref': visit.referrer, 'ua': visit.user_agent.encode('utf-8'), 'cip': visit.remote_ip, 'cdt': visit.datetime.strftime("%Y-%m-%d %H:%M:%S"),
                            'idsite': settings.PIWIK_ID, 'rec': 1, 'token_auth': settings.PIWIK_TOKEN, 'action_name': url.url.encode('utf-8'), 'apiv': 1})

            res = '?' + urlencode(params)
            visits.append(res)
            visit.tracked = True
            visit.save()

        payload = {'requests': visits, 'token_auth': settings.PIWIK_TOKEN}
        headers = {'content-type': 'application/json'}
        requests.post(settings.PIWIK_URL, data=json.dumps(payload), headers=headers)
        print('All tracked well!')
