from django.db import models
from django.db.models.signals import post_save
from django.dispatch import receiver


class URL(models.Model):
    url = models.URLField(max_length=512, help_text='Maximum 512 characters.', db_index=True)
    slug = models.SlugField(unique=True, help_text="Must be unique. Contains what will be in URL after http://smrdim.cz/", null=True)
    campaign_name = models.CharField(default='', blank=True, max_length=100)
    campaign_keyword = models.CharField(default='', blank=True, max_length=100)
    datetime = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('-id',)

    def __unicode__(self):
        return self.slug

    @models.permalink
    def get_absolute_url(self):
        return 'short-url', (), {'slug': self.url}

    def update_url(self):
        self.slug = get_url(self.id)
        self.save()


def get_url(number):
    string = "1aqzxsw23edcvfr45tgbnhy67ujmki89olp0QAZXRSWEDCVFTGBNHYUJMKIOLP"
    length = len(string)
    out = ""

    if number == 0:
        return string[0]

    while number != 0:
        i = number % length
        number /= length
        out = string[i] + out

    return out


# auto update url on first save
@receiver(post_save, sender=URL)
def update_url(created, instance, **kwargs):
    if created:
        instance.update_url()


class Visit(models.Model):
    url = models.ForeignKey(URL, related_name='visits')
    referrer = models.URLField(blank=True, max_length=512)
    user_agent = models.TextField()
    cookie = models.CharField(max_length=127, blank=True, default='')
    remote_ip = models.GenericIPAddressField()
    datetime = models.DateTimeField(auto_now_add=True)
    tracked = models.BooleanField(default=False)

    class Meta:
        ordering = ('-datetime',)


class API_IP(models.Model):
    count = models.PositiveIntegerField(default=0)
    remote_ip = models.GenericIPAddressField()
    start = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('-start',)

    def __unicode__(self):
        return '%s' % self.remote_ip
